package com.gic.haoban.manage.service.service.impl;

import static org.slf4j.LoggerFactory.getLogger;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.JSONResponse;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.MaterialDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanMaterialMapper;
import com.gic.haoban.manage.service.dao.mapper.TabMaterialContentMapper;
import com.gic.haoban.manage.service.entity.TabHaobanMaterial;
import com.gic.haoban.manage.service.entity.TabMaterialContent;
import com.gic.haoban.manage.service.service.MaterialService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.util.CommonUtil;
import com.gic.wechat.api.enums.QywxMediaTypeEnum;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.github.pagehelper.Page;

@Service
public class MaterialServiceImpl implements MaterialService {
	
    private static final Logger logger = getLogger(MaterialService.class);
    
	@Autowired
	private TabHaobanMaterialMapper mapper;
	@Autowired
	private TabMaterialContentMapper materialContentMapper ;
	@Autowired
	private WxEnterpriseService wxEnterpriseService ;
	@Autowired
	private QywxSuiteApiService qywxSuiteApiService ;
    @Autowired
    private Config config;

	@Override
	public List<MaterialDTO> listMaterialByCategoryId(String categoryId) {
		return EntityUtil.changeEntityListByJSON(MaterialDTO.class, mapper.listMaterialByCategoryId(categoryId));
	}

	@Override
	public void insertMaterial(MaterialDTO materialDTO) {
		materialDTO.setCreateTime(new Date());
		materialDTO.setUpdateTime(materialDTO.getCreateTime());
		materialDTO.setStatusFlag(1);
		materialDTO.setMaterialId(StringUtil.randomUUID());
		mapper.insert(EntityUtil.changeEntityByJSON(TabHaobanMaterial.class, materialDTO));
		
	}

	@Override
	public void batchInsertMaterial(List<MaterialDTO> materialList) {
		mapper.batchInsert(EntityUtil.changeEntityListByOrika(TabHaobanMaterial.class, materialList));
	}

	@Override
	public MaterialDTO selectMaterialById(String materialId) {
		return EntityUtil.changeEntityByJSON(MaterialDTO.class, mapper.selectByPrimaryKey(materialId));
	}

	@Override
    public List<MaterialDTO> listMaterialByIds(List<String> materialIds) {
        return EntityUtil.changeEntityListByOrika(MaterialDTO.class, mapper.listByids(materialIds));
    }

    @Override
    public void edit(MaterialDTO materialDTO) {
        materialDTO.setUpdateTime(new Date());
        mapper.updateByPrimaryKeySelective(EntityUtil.changeEntityByJSON(TabHaobanMaterial.class, materialDTO));
    }

	@Override
	public Page<TabHaobanMaterial> listMaterial(String wxEnterpriseId, String keyword, String categoryId, Integer materialType) {
		return mapper.listMaterial(wxEnterpriseId,keyword,categoryId,materialType);
	}

	@Override
	public Page<TabHaobanMaterial> listMaterial(String wxEnterpriseId, String keyword, List<String> categoryIds, Integer materialType) {
		List<TabHaobanMaterial> materials = mapper.listMaterialByCategoryIds(wxEnterpriseId, keyword, categoryIds, materialType);
		return (Page) materials;
	}

	@Override
	public MaterialDTO getMaterialByFromMaterialId(String fromMaterialId, String wxEnterpriseId) {
        TabHaobanMaterial tabHaobanMaterial = mapper.selectByfromMaterialId(fromMaterialId, wxEnterpriseId);
        return EntityUtil.changeEntityByJSON(MaterialDTO.class, tabHaobanMaterial);
    }

	@Override
	public List<MaterialDTO> listMaterialByFromMateralId(String fromMaterialId) {
		List<TabHaobanMaterial> list = mapper.listByfromMaterialId(fromMaterialId);
		return EntityUtil.changeEntityListByOrika(MaterialDTO.class, list);
	}

	@Override
	public void delmaterialByIds(List<String> ids) {
		mapper.delMaterialByIds(ids);
	}
	
	@Override
	public MaterialDTO getHasChangeMadieMaterialById(String materialId, int from) {
	    MaterialDTO materialDTO = null ;
        TabMaterialContent entity =null ;
        if(from == 1) {
            materialDTO = this.selectMaterialById(materialId);
        }
        if(from == 2) {
            entity =this.materialContentMapper.selectById(materialId) ;
            if(null != entity) {
                materialDTO = EntityUtil.changeEntityByJSON(MaterialDTO.class, entity) ;
            }
        }
        if (null == materialDTO) {
            return null;
        }
        Date lastUploadTime = materialDTO.getWxLastUploadTime();
        Date date = new Date();
        Date compareDate = DateUtils.addDays(date, -3);
        if (lastUploadTime != null && lastUploadTime.after(compareDate) && StringUtils.isNotBlank(materialDTO.getMediaId())) {
            return materialDTO;
        }
        Integer materialType = materialDTO.getMaterialType();
        String madiaId = materialDTO.getMediaId();
        //需要上传的类型
        List<Integer> typeList = Arrays.asList(2, 4, 5, 6);
        if (typeList.contains(materialType)) {
            madiaId = this.reUpdalodMetail(materialId,from);
        }
        materialDTO.setMediaId(madiaId);
        materialDTO.setWxLastUploadTime(date);
        return materialDTO;
        
	}
	
	  private String reUpdalodMetail(String materialId,int from) {
	        MaterialDTO old = null ;
	        TabMaterialContent entity =null ;
	        if(from == 1) {
	            old = this.selectMaterialById(materialId);
	        }
	        if(from == 2) {
	            entity =this.materialContentMapper.selectById(materialId) ;
	            if(null != entity) {
	                old = EntityUtil.changeEntityByJSON(MaterialDTO.class, entity) ;
	            }
	        }
	        Integer type = old.getMaterialType();
	        if (type == null) {
	            return "";
	        }
	        String wxEnterpriseId = old.getWxEnterpriseId();
	        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId) ;
	        if (qwDTO == null) {
	            logger.info("qwDTO is null");
	            return "";
	        }
	        String url = "";
	        QywxMediaTypeEnum fileType = null;
	        if (type == 2) {
	            fileType = QywxMediaTypeEnum.IMAGE;
	            url = old.getImgUrl();
	        }
	        if (type == 4) {
	            fileType = QywxMediaTypeEnum.VIDEO;
	            url = old.getLink();
	        }
	        if (type == 5) {
	            fileType = QywxMediaTypeEnum.FILE;
	            url = old.getLink();
	        } else if (type == 6) {
	            fileType = QywxMediaTypeEnum.IMAGE;
	            url = old.getImgUrl();
	        }
	        if (fileType == null) {
	            logger.error("materialId无类型,materialId:{}", materialId);
	            return "";
	        }
	        String[] arr = url.split("/");
	        int count = arr.length;
	        JSONResponse jp = null ;
	        if(from==2 && entity.getMediaType()==2) {
	            jp = qywxSuiteApiService.uploadAttachment(qwDTO.getThirdCorpid(), config.getWxSuiteid(), CommonUtil.getFileByte(url), arr[count - 1], fileType.getCode());
	        }else {
	            jp = qywxSuiteApiService.uploadMedia(qwDTO.getThirdCorpid(), config.getWxSuiteid(), CommonUtil.getFileByte(url), arr[count - 1], fileType.getCode());
	        }
	        String mediaid = old.getMediaId();
	        if (jp.getErrorCode() == 0) {
	            if(from==1) {
	                old.setWxLastUploadTime(new Date());
	                old.setMediaId(jp.getResult().toString());
	                mediaid = old.getMediaId() ;
	                this.edit(old);
	            }
	            if(from==2) {
	                entity.setWxLastUploadTime(new Date());
	                entity.setMediaId(jp.getResult().toString());
	                mediaid = entity.getMediaId() ;
	                this.materialContentMapper.updateByPrimaryKey(entity) ;
	            }
	        }
	        return mediaid;
	    }
	
}
