package com.gic.haoban.manage.service.service.impl;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.commons.util.EntityUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.service.StaffService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

@Service
public class StaffServiceImpl implements StaffService {
    private static final Logger log = LogManager.getLogger(StaffServiceImpl.class);

    @Autowired
    private StaffMapper mapper;
    @Override
    public TabHaobanStaff selectById(String id) {
        if (StringUtils.isBlank(id)) {
            return null;
        }
        return mapper.selectByPrimaryKey(id);
    }

    @Override
    public TabHaobanStaff selectByNationcodeAndPhoneNumber(String wxEnterpriseId, String nationcode, String phoneNumber) {
        if (StringUtils.isBlank(phoneNumber)) {
            return null;
        }
        return mapper.selectByNationcodeAndPhoneNumber(wxEnterpriseId, nationcode, phoneNumber);
    }

    @Override
    public String add(TabHaobanStaff tab) {
        Date now = new Date();
        tab.setCreateTime(now);
        tab.setUpdateTime(now);
        if (StringUtils.isBlank(tab.getStaffId())) {
            tab.setStaffId(StringUtil.randomUUID());
        }
        tab.setStatusFlag(1);
        tab.setAddNum(0);
        mapper.insertSelective(tab);
        return tab.getStaffId();
    }

    @Override
    public Page<TabHaobanStaff> pageStaff(List<String> staffIds, Integer activeFlag, String keyword,List<String> departmentIds,String wxEnterpriseId,Integer relationFlag) {
        return mapper.pageStaff(staffIds, activeFlag, keyword,departmentIds,wxEnterpriseId,relationFlag);
    }

    @Override
    public TabHaobanStaff selectByUserIdAndEnterpriseId(String userId, String wxEnterpriseId) {
        return mapper.selectByUserIdAndEnterpriseId(userId, wxEnterpriseId);
    }

    @Override
    public TabHaobanStaff selectByUserIdAndEnterpriseId(String wxUserId, String wxOpenUserId , String wxEnterpriseId) {
        if (StringUtils.isBlank(wxUserId)) {
            log.info("wxUserId is null");
            return null;
        }
        TabHaobanStaff staff = this.mapper.selectByUserIdAndEnterpriseId(wxUserId, wxEnterpriseId) ;
        if (null == staff && null != wxOpenUserId && !wxUserId.equals(wxOpenUserId)) {
            return  this.mapper.selectByUserIdAndEnterpriseId(wxOpenUserId, wxEnterpriseId) ;
        }
        return staff;
    }

    @Override
    public void updateByPrimaryKey(TabHaobanStaff tab) {
        tab.setUpdateTime(new Date());
        mapper.updateByPrimaryKeySelective(tab);
    }

    @Override
    public int delByStaffId(String staffId) {
        TabHaobanStaff staff = new TabHaobanStaff();
        staff.setStaffId(staffId);
        staff.setStatusFlag(0);
        return this.mapper.updateByPrimaryKeySelective(staff);
    }

    @Override
    public List<StaffDTO> listByUserIdsAndWxEnterpriseId(List<String> userIds, String wxEnterpriseId) {
        return EntityUtil.changeEntityListByJSON(StaffDTO.class, this.mapper.listByUserIdsAndWxEnterpriseId(userIds, wxEnterpriseId));
    }

    @Override
    public int cleanStaff(String wxEnterpriseId, List<String> staffIds) {
        return mapper.cleanStaff(wxEnterpriseId, staffIds);
    }

    @Override
    public List<String> listDelUserStaffId(String wxEnterpriseId, List<String> staffIds) {
        return mapper.listUserStaffId(wxEnterpriseId, staffIds);
    }

    @Override
    public List<TabHaobanStaff> listNoStatusByStaffIds(String wxEnterpriseId, List<String> staffIds) {
        return mapper.listNoStatusStaffIds(wxEnterpriseId, staffIds);
    }

    @Override
    public List<StaffDTO> listByOpenUserIdsAndWxEnterpriseId(List<String> openUserIds, String wxEnterpriseId) {
        return EntityUtil.changeEntityListByJSON(StaffDTO.class, mapper.listByOpenUserIdsAndWxEnterpriseId(openUserIds, wxEnterpriseId));
    }

    @Override
    public int updateOpenUserIdsByUserId(String wxEnterpriseId, String wxUserId, String wxOpenUseId) {
        return mapper.updateOpenUserIdsByUserId(wxEnterpriseId, wxUserId, wxOpenUseId);
    }

    @Override
    public void delOtherStaffByWxUserId(String wxUserId, String staffId, String wxEnterpriseId) {
        mapper.delOtherStaffByWxUserId(wxUserId, staffId, wxEnterpriseId);
    }

    @Override
    public com.gic.api.base.commons.Page<StaffDTO> getPageStaffIdsByWxEnterpriseId(String wxEnterpriseId, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        List<TabHaobanStaff> tabHaobanStaffs = mapper.listByWxEnterpriseId(wxEnterpriseId);
        return PageUtil.changePageHelperToCurrentPage(new PageInfo<>(tabHaobanStaffs),StaffDTO.class);
    }

    @Override
    public int updateAddNumById(String staffId) {
        return mapper.updateAddNumById(staffId);
    }

    @Override
    public int resetAddNum() {
        return mapper.resetAddNum();
    }

    @Override
    public int updateActiveStatusById(Integer activeStatus, String staffId) {
        return mapper.updateActiveStatusById(activeStatus,staffId);
    }
    
    @Override
    public void updateOpenConcatFlag(Set<String> staffIdList) {
        this.mapper.updateOpenConcatFlag(staffIdList);
    }
}
