package com.gic.haoban.manage.service.service.out.impl.chat;

import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.haoban.manage.api.dto.chat.ChatEventDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatOwnerDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatOwnerHistoryDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatUserDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.GroupChatSearchQDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.GroupChatUserSearchQDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.OwnerSearchQDTO;
import com.gic.haoban.manage.api.service.chat.GroupChatApiService;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterpriseRelated;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatBO;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WxEnterpriseRelatedService;
import com.gic.haoban.manage.service.service.chat.GroupChatService;
import com.gic.haoban.manage.service.service.chat.GroupChatUserService;

@Service("groupChatApiService")
public class GroupChatApiServiceImpl implements GroupChatApiService {

    private static final Logger logger = LogManager.getLogger(GroupChatApiService.class);

    @Autowired
    private GroupChatService groupChatService;
    @Autowired
    private GroupChatUserService groupChatUserService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private WxEnterpriseRelatedService wxEnterpriseRelatedService;
    @Autowired
    private StaffClerkRelationService staffClerkRelationService;

    @Override
    public ServiceResponse<Page<GroupChatDTO>> listPage(GroupChatSearchQDTO qdto, BasePageInfo basePageInfo) {
        if(null != qdto.getEndDate()) {
            qdto.setEndDate(DateUtil.getEndTimeOfDay(qdto.getEndDate()));
        }
        String sortColumn = qdto.getSortColumn();
        if (StringUtils.isBlank(sortColumn)) {
            sortColumn = null;
        } else if ("totalCount".equals(sortColumn)) {
            sortColumn = "total_count";
        } else if ("totalMemberCount".equals(sortColumn)) {
            sortColumn = "total_member_count";
        } else if ("addMemberCount".equals(sortColumn)) {
            sortColumn = "add_member_count";
        } else if ("addCount".equals(sortColumn)) {
            sortColumn = "add_count";
        } else if ("quitCount".equals(sortColumn)) {
            sortColumn = "quit_count";
        } else if ("quitMemberCount".equals(sortColumn)) {
            sortColumn = "quit_member_count";
        }
        qdto.setSortColumn(sortColumn);
        Page<GroupChatDTO> page = this.groupChatService.listPage(qdto, basePageInfo);
        Page<GroupChatDTO> resultPage = PageHelperUtils.changePageToCurrentPage(page, GroupChatDTO.class);
        List<GroupChatDTO> dtoList = resultPage.getResult();
        // 处理关联状态
        if (CollectionUtils.isNotEmpty(dtoList)) {
            boolean needClerkReltaion = (qdto.getStatus() == 1);
            if (needClerkReltaion) {
                List<String> staffIdList = dtoList.stream().filter(dto -> StringUtils.isNotBlank(dto.getStaffId()))
                        .map(dto -> dto.getStaffId()).collect(Collectors.toList());
                List<String> relationIdList = this.staffClerkRelationService
                        .listRelationsStaffId(new HashSet<>(staffIdList));
                dtoList.forEach(one -> {
                    one.setClerkRelationFlag(relationIdList.contains(one.getStaffId()) ? 1 : 0);
                });
            }
            // 原群主
            if (qdto.getStatus() == 2) {
                dtoList.forEach(one -> {
                    String staffId = one.getOriginalStaffId();
                    TabHaobanStaff staff = this.staffService.selectById(staffId);
                    if (null != staff) {
                        one.setOriginalStaffName(staff.getStaffName());
                    }
                });
            }
        }

        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Page<GroupChatUserDTO>> listUserPage(GroupChatUserSearchQDTO qdto,
            BasePageInfo basePageInfo) {
        if(null != qdto.getEndDate()) {
            qdto.setEndDate(DateUtil.getEndTimeOfDay(qdto.getEndDate()));
        }
        Page<GroupChatUserDTO> page = this.groupChatUserService.listPage(qdto, basePageInfo);
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<GroupChatDTO> detail(String wxEnterpriseId, Long groupChatId) {
        GroupChatBO bo = this.groupChatService.detail(wxEnterpriseId, groupChatId);
        GroupChatDTO dto = EntityUtil.changeEntityByJSON(GroupChatDTO.class, bo);
        if (StringUtils.isNotBlank(dto.getStaffId())) {
            TabHaobanStaff staff = this.staffService.selectById(dto.getStaffId());
            if (null != staff) {
                dto.setStaffName(staff.getStaffName());
                dto.setStaffImage(staff.getHeadImg());
            }
        }
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<Void> refreshChatInfo(Long chatHmId) {
        this.groupChatService.refreshChatInfo(chatHmId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> refreshWxEnterpriseChatInfo(String wxEnterpriseId) {
        this.groupChatService.initEnterpriseGroupChat(wxEnterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateChatEnterpriseId(String wxEnterpriseId, String enterpriseId,
            List<Long> groupChatIdList) {
        this.groupChatService.updateChatEnterpriseId(wxEnterpriseId, enterpriseId, groupChatIdList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<GroupChatOwnerDTO>> listOwnerPage(OwnerSearchQDTO qdto, BasePageInfo basePageInfo) {
        Page<GroupChatOwnerDTO> page = this.groupChatService.listOwnerPage(qdto, basePageInfo);
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Void> qwEventNotice(ChatEventDTO dto) {
        logger.info("收到群事件,params={}", JSON.toJSONString(dto));
        String changeType = dto.getChangeType();
        String wxChatId = dto.getChatId();
        String corpid = dto.getCorpid();
        // 群解散事件
        if ("dismiss".equals(changeType)) {
            this.groupChatService.dismissChat(corpid, wxChatId);
        } else if ("create".equals(changeType)) {
            // 创建群
            this.groupChatService.createChat(corpid, wxChatId);
        } else {
            String updateDetail = dto.getUpdateDetail();
            this.groupChatService.updateChat(corpid, wxChatId, updateDetail, JSON.toJSONString(dto));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<GroupChatOwnerHistoryDTO>> listOwnerHistory(Long groupChatId, String WxEnterpriseId) {
        return ServiceResponse.success(this.groupChatService.listOwnerHistory(groupChatId, WxEnterpriseId));
    }

    @Override
    public ServiceResponse<Void> transfer(String wxEnterpriseId, String staffId, List<Long> groupChatIdList) {
        this.groupChatService.transfer(wxEnterpriseId, staffId, groupChatIdList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Boolean> getSetGicFlag(String wxEnterpriseId) {
        List<TabHaobanWxEnterpriseRelated> list = this.wxEnterpriseRelatedService.getByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtils.isNotEmpty(list) && list.size() == 1) {
            return ServiceResponse.success(Boolean.TRUE);
        }
        return ServiceResponse.success(Boolean.FALSE);
    }

    @Override
    public ServiceResponse<Void> refreshChatInfoFromMQ(String params) {
        JSONObject json = JSONObject.parseObject(params);
        List<Long> arr = json.parseArray(json.getString("groupChatIdList"), Long.class);
        for (Long id : arr) {
            this.refreshChatInfo(id);
        }
        return ServiceResponse.success();
    }
}
