package com.gic.haoban.manage.service.service.out.impl.chat;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.haoban.manage.api.dto.chat.GroupChatHmDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkChatDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.ChatHmSearchQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkChatQDTO;
import com.gic.haoban.manage.api.service.chat.GroupChatHmApiService;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatHmBO;
import com.gic.haoban.manage.service.service.chat.GroupChatHmService;
import com.gic.haoban.manage.service.service.hm.HmLinkChatService;

/**
 * 
* @ClassName: GroupChatHmApiService
* @Description: 群活码
* @author xugh
* @date 2022年12月1日 下午7:44:51
*
 */
@Service("groupChatHmApiService")
public class GroupChatHmApiServiceImpl implements GroupChatHmApiService {

    @Autowired
    private GroupChatHmService groupChatHmService;
    @Autowired
    private HmLinkChatService hmLinkChatService;

    @Override
    public ServiceResponse<Page<GroupChatHmDTO>> listPage(ChatHmSearchQDTO qdto, BasePageInfo basePageInfo) {
        Page<GroupChatHmBO> page = this.groupChatHmService.listPage(qdto, basePageInfo);
        Page<GroupChatHmDTO> resultPage = PageHelperUtils.changePageToCurrentPage(page, GroupChatHmDTO.class);
        if(CollectionUtils.isNotEmpty(resultPage.getResult())) {
            List<Long> chatHmIdList = resultPage.getResult().stream().map(dto -> dto.getChatHmId())
                    .collect(Collectors.toList());
            List<HmLinkChatDTO> linkCount = this.hmLinkChatService.getLinkCountForHm(qdto.getWxEnterpriseId(),
                    chatHmIdList);
            Map<Long, Integer> map = linkCount.stream()
                    .collect(Collectors.toMap(HmLinkChatDTO::getChatHmId, HmLinkChatDTO::getLinkCount, (k1, k2) -> k1));

            resultPage.getResult().forEach(dto -> {
                if (null != map.get(dto.getChatHmId())) {
                    dto.setLinkCount(map.get(dto.getChatHmId()));
                } else {
                    dto.setLinkCount(0);
                }
            });
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<GroupChatHmDTO> detail(Long chatHmId) {
        GroupChatHmBO bo = this.groupChatHmService.getById(chatHmId);
        GroupChatHmDTO dto = EntityUtil.changeEntityByJSON(GroupChatHmDTO.class, bo);
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<GroupChatHmDTO> save(GroupChatHmDTO dto) {
        this.groupChatHmService.saveHm(dto);
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<Void> discard(Long chatHmId, String wxEnterpriseId, String clerkId, String clerkName) {
        this.groupChatHmService.discard(chatHmId, wxEnterpriseId, clerkId, clerkName);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<HmLinkChatDTO>> listPageChatLink(HmLinkChatQDTO qdto, BasePageInfo basePageInfo) {
        Page<HmLinkChatDTO> page = this.hmLinkChatService.listPage(qdto, basePageInfo);
        return ServiceResponse.success(page);
    }

}
