package com.gic.haoban.manage.web.controller.chat;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.JSONResponse;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.AuthorizedUser;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.ToolUtil;
import com.gic.commons.web.RequestThreadLocal;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.api.dto.security.DownloadReportDTO;
import com.gic.enterprise.api.service.DownloadReportService;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.manage.api.dto.chat.GroupChatDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatHmDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkChatDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.ChatHmSearchQDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.GroupChatSearchQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkChatQDTO;
import com.gic.haoban.manage.api.service.chat.GroupChatApiService;
import com.gic.haoban.manage.api.service.chat.GroupChatHmApiService;
import com.gic.haoban.manage.web.utils.CreatorUtils;
import com.gic.haoban.manage.web.vo.chat.HmLinkChatListVO;
import com.gic.qcloud.BucketNameEnum;
import com.gic.qcloud.FileUploadUtil;
import com.gic.web.common.utils.DataPermissionExport;
import com.gic.web.common.utils.ExecutorPoolSingleton;
import com.gic.web.common.utils.SessionContextUtils;

/**
 * 
* @ClassName: GroupChatHmController
* @Description: 群活码
* @author xugh
* @date 2022年11月26日 上午9:32:56
*
 */
@RestController
@RequestMapping("/chat/hm")
public class GroupChatHmController {

    @Autowired
    private GroupChatHmApiService groupChatHmApiService;
    @Autowired
    private GroupChatApiService groupChatApiService;
    @Autowired
    private DownloadReportService downloadReportService;

    @RequestMapping("add")
    public RestResponse<Object> save(@RequestBody GroupChatHmDTO dto) {
        if (CollectionUtils.isEmpty(dto.getChatIdList())) {
            return RestResponse.failure("1", "请选择加入的群");
        }
        if (dto.getChatIdList().size() > 4) {
            return RestResponse.failure("1", "最多可选4个群");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        dto.setCreatorId(loginUser.getClerkId());
        dto.setCreatorName(loginUser.getClerkName());
        dto.setModifierId(loginUser.getClerkId());
        dto.setModifierName(loginUser.getClerkName());
        dto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        dto.setEnterpriseId(loginUser.getEnterpriseId());
        this.groupChatHmApiService.save(dto);
        return RestResponse.successResult();
    }

    @RequestMapping("update")
    public RestResponse<Object> update(@RequestBody GroupChatHmDTO dto) {
        if (CollectionUtils.isEmpty(dto.getChatIdList())) {
            return RestResponse.failure("1", "请选择加入的群");
        }
        if (dto.getChatIdList().size() > 4) {
            return RestResponse.failure("1", "最多可选4个群");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        dto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        dto.setEnterpriseId(loginUser.getEnterpriseId());
        dto.setModifierId(loginUser.getClerkId());
        dto.setModifierName(loginUser.getClerkName());
        this.groupChatHmApiService.save(dto);
        return RestResponse.successResult();
    }

    @RequestMapping("page")
    public RestResponse<Object> page(ChatHmSearchQDTO qdto, BasePageInfo basePageInfo) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        qdto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        qdto.setEnterpriseId(loginUser.getEnterpriseId());
        List<String> creatorIdList = CreatorUtils.getCreatorIdList(0);
        if (CollectionUtils.isNotEmpty(creatorIdList)) {
            qdto.setCreatorIdList(creatorIdList);
        }
        ServiceResponse<Page<GroupChatHmDTO>> page = this.groupChatHmApiService.listPage(qdto, basePageInfo);
        return RestResponse.successResult(page.getResult());
    }

    @RequestMapping("detail")
    public RestResponse<Object> detail(Long chatHmId) {
        if (null == chatHmId) {
            return RestResponse.failure("1", "群活码ID空");
        }
        ServiceResponse<GroupChatHmDTO> linkResp = this.groupChatHmApiService.detail(chatHmId);
        if (!linkResp.isSuccess()) {
            return RestResponse.failure("1", "群活码不存在");
        }
        GroupChatHmDTO hm = linkResp.getResult();
        return RestResponse.successResult(hm);
    }

    @RequestMapping("discard")
    public RestResponse<Object> discard(Long chatHmId) {
        if (null == chatHmId) {
            return RestResponse.failure("1", "群活码ID空");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        String clerkId = loginUser.getClerkId();
        String clerkName = loginUser.getClerkName();
        this.groupChatHmApiService.discard(chatHmId, wxEnterpriseId, clerkId, clerkName);
        return RestResponse.successResult();
    }

    /**
     * 
    * @Title: linkRelation
    * @Description: 查询使用此活码的链接列表
    * @author xugh
    * @param chatHmId
    * @param basePageInfo
    * @return
    * @throws
     */
    @RequestMapping("link-relation")
    public RestResponse<Object> linkRelation(Long chatHmId, BasePageInfo basePageInfo) {
        if (null == chatHmId) {
            return RestResponse.failure("1", "群活码ID空");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        HmLinkChatQDTO qdto = new HmLinkChatQDTO();
        qdto.setWxEnterpriseId(wxEnterpriseId);
        qdto.setEnterpriseId(loginUser.getEnterpriseId());
        qdto.setChatHmId(chatHmId);
        ServiceResponse<Page<HmLinkChatDTO>> pageResp = this.groupChatHmApiService.listPageChatLink(qdto, basePageInfo);
        Page<HmLinkChatDTO> page = pageResp.getResult();
        Page<HmLinkChatListVO> retPage = PageHelperUtils.changePageToCurrentPage(page, HmLinkChatListVO.class);
        return RestResponse.successResult(retPage);
    }

    /**
     * 
    * @Title: groupRelation
    * @Description: 查询群活码关联的群列表
    * @author xugh
    * @param chatHmId
    * @param basePageInfo
    * @return
    * @throws
     */
    @RequestMapping("group-relation")
    public RestResponse<Object> groupRelation(Long chatHmId, BasePageInfo basePageInfo) {
        if (null == chatHmId) {
            return RestResponse.failure("1", "群活码ID空");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        GroupChatSearchQDTO qdto = new GroupChatSearchQDTO();
        qdto.setWxEnterpriseId(wxEnterpriseId);
        qdto.setEnterpriseId(loginUser.getEnterpriseId());
        qdto.setChatHmId(chatHmId);
        ServiceResponse<Page<GroupChatDTO>> page = this.groupChatApiService.listPage(qdto, basePageInfo);
        return RestResponse.successResult(page.getResult());
    }

    @SuppressWarnings("deprecation")
    @RequestMapping("download")
    public RestResponse<Object> download(ChatHmSearchQDTO qdto, @RequestParam(defaultValue = "0") int allFlag) {
        if (allFlag == 0 && CollectionUtils.isEmpty(qdto.getChatHmIdList())) {
            return RestResponse.failure("1", "请选择要下载的群活码");
        }
        if (allFlag == 1) {
            qdto.setChatHmIdList(null);
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String qrCodeName = "群活码";
        final String fileName = qrCodeName + "_" + sdf.format(new Date()) + ".zip";
        final AuthorizedUser au = SessionContextUtils.getLoginUser();
        String path = RequestThreadLocal.get().getSession().getServletContext().getRealPath("/chat/qrcode/");
        String uuId = ToolUtil.randomUUID();
        path = path + uuId;
        final String tempPath = path + File.separator + fileName;

        File file = new File(tempPath);
        if (!file.exists()) {
            file.getParentFile().mkdirs();
            try {
                file.createNewFile();
            } catch (IOException e) {
            }
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        String enterpriseId = loginUser.getEnterpriseId();
        BasePageInfo basePageInfo = new BasePageInfo();
        basePageInfo.setPageNum(1);
        basePageInfo.setPageSize(Integer.MAX_VALUE);
        qdto.setEnterpriseId(enterpriseId);
        qdto.setWxEnterpriseId(wxEnterpriseId);
        ServiceResponse<Page<GroupChatHmDTO>> resp = this.groupChatHmApiService.listPage(qdto, basePageInfo);
        List<GroupChatHmDTO> dtoList = resp.getResult().getResult();
        int size = dtoList.size();
        JSONResponse jsonResponse = DataPermissionExport.generatedExcel(1, au, "", "", size, fileName, qrCodeName,
                downloadReportService);
        final String reportId;
        if (jsonResponse.isSuccess()) {
            reportId = (String) jsonResponse.getResult();
        } else {
            return RestResponse.failure("1", jsonResponse.getMessage());
        }
        ExecutorPoolSingleton.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    File tempFile = new File(tempPath);
                    ZipOutputStream out = new ZipOutputStream(new FileOutputStream(tempFile));
                    for (GroupChatHmDTO dto : dtoList) {
                        String qrCodeUrl = dto.getWxQrCode();
                        if (StringUtils.isNotBlank(qrCodeUrl)) {
                            try {
                                ZipEntry e = new ZipEntry(dto.getChatHmCode() + "_" + dto.getName() + ".png");
                                out.putNextEntry(e);
                                out.write(getStoreQrCode(qrCodeUrl));
                                out.flush();
                            } catch (IOException e) {
                            }
                        }
                    }
                    try {
                        out.flush();
                        out.closeEntry();
                        out.close();
                    } catch (IOException e) {
                    }
                    String url = FileUploadUtil.simpleUploadFileFromLocal(tempFile, fileName,
                            BucketNameEnum.COMPRESS_60000.getName());
                    DownloadReportDTO downloadReportDTO = new DownloadReportDTO();
                    downloadReportDTO.setDownloadUrl("https://" + url);
                    downloadReportService.updateDownloadReport(reportId, downloadReportDTO);
                    tempFile.deleteOnExit();
                } catch (Exception e) {
                }
            }
        });
        return RestResponse.successResult(reportId);
    }

    private byte[] getStoreQrCode(String qrCodeUrl) {
        HttpURLConnection conn = null;
        try {
            URL url = new URL(qrCodeUrl);
            try {
                conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                InputStream in = conn.getInputStream();
                return IOUtils.toByteArray(in);
            } catch (IOException e) {
                return null;
            }
        } catch (MalformedURLException e) {
            return null;
        }
    }

}
