package com.gic.haoban.manage.web.controller.chat;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.manage.api.dto.MaterialDTO;
import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanLogDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanOwnerLogDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.GroupChatPlanSearchQDTO;
import com.gic.haoban.manage.api.service.MaterialApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.chat.GroupChatPlanApiService;
import com.gic.haoban.manage.web.utils.CreatorUtils;
import com.gic.haoban.manage.web.vo.chat.ChatOwnerVO;
import com.gic.haoban.manage.web.vo.chat.GroupChatPlanVO;

/**
 * 
* @ClassName: GroupChatPlanController
* @Description: 群群发计划
* @author xugh
* @date 2022年11月26日 上午9:32:56
*
 */
@RestController
@RequestMapping("/chat/plan")
public class GroupChatPlanController {

    private static org.apache.logging.log4j.Logger logger = LogManager.getLogger(GroupChatPlanController.class);

    @Autowired
    private GroupChatPlanApiService groupChatPlanApiService;
    @Autowired
    private MaterialApiService materialApiService;
    @Autowired
    private StaffApiService staffApiService;

    @RequestMapping("add")
    public RestResponse<Object> save(@RequestBody GroupChatPlanDTO dto) {
        if (StringUtils.isNotBlank(dto.getStaffIdList())) {
            return RestResponse.failure("9999", "请选择群主");
        }
        logger.info("新增参数={}", JSON.toJSONString(dto));
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        dto.setCreatorId(loginUser.getClerkId());
        dto.setCreatorName(loginUser.getClerkName());
        dto.setModifierId(loginUser.getClerkId());
        dto.setModifierName(loginUser.getClerkName());
        dto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        dto.setEnterpriseId(loginUser.getEnterpriseId());
        this.groupChatPlanApiService.save(dto);
        return RestResponse.successResult();
    }

    @RequestMapping("del")
    public RestResponse<Object> del(Long planId) {
        if (null == planId) {
            return RestResponse.failure("9999", "计划ID空");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        this.groupChatPlanApiService.del(loginUser.getClerkId(), loginUser.getClerkName(), planId);
        return RestResponse.successResult();
    }

    /**
     * 
    * @Title: stop
    * @Description: 终止
    * @author xugh
    * @param planId
    * @return
    * @throws
     */
    @RequestMapping("stop")
    public RestResponse<Object> stop(Long planId) {
        if (null == planId) {
            return RestResponse.failure("9999", "计划ID空");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        this.groupChatPlanApiService.stop(loginUser.getClerkId(), loginUser.getClerkName(), planId);
        return RestResponse.successResult();
    }

    @RequestMapping("update")
    public RestResponse<Object> update(@RequestBody GroupChatPlanDTO dto) {
        logger.info("更新参数={}", JSON.toJSONString(dto));
        ServiceResponse<GroupChatPlanDTO> resp = this.groupChatPlanApiService.detail(dto.getPlanId());
        GroupChatPlanDTO plan = resp.getResult();
        if (null == plan) {
            return RestResponse.failure("9999", "计划不存在");
        }
        if (plan.getSendType() == 1) {
            return RestResponse.failure("9999", "已发送的不能编辑");
        }
        if (null != plan.getSendTime() && plan.getSendTime().before(new Date())) {
            return RestResponse.failure("9999", "已发送的不能编辑");
        }
        if (plan.getStartTime().before(new Date())) {
            return RestResponse.failure("9999", "进行中的不能编辑");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        dto.setModifierId(loginUser.getClerkId());
        dto.setModifierName(loginUser.getClerkName());
        this.groupChatPlanApiService.save(dto);
        return RestResponse.successResult();
    }

    @RequestMapping("page")
    public RestResponse<Object> page(GroupChatPlanSearchQDTO qdto, BasePageInfo basePageInfo) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        qdto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        qdto.setEnterpriseId(loginUser.getEnterpriseId());
        List<String> creatorIdList = CreatorUtils.getCreatorIdList(0);
        if (CollectionUtils.isNotEmpty(creatorIdList)) {
            qdto.setCreatorIdList(creatorIdList);
        }
        ServiceResponse<Page<GroupChatPlanDTO>> page = this.groupChatPlanApiService.listPage(qdto, basePageInfo);
        return RestResponse.successResult(page.getResult());
    }

    @RequestMapping("detail")
    public RestResponse<Object> detail(Long planId, @RequestParam(defaultValue = "0") int updteFlag) {
        ServiceResponse<GroupChatPlanDTO> linkResp = this.groupChatPlanApiService.detail(planId);
        if (!linkResp.isSuccess()) {
            return RestResponse.failure("1", "计划不存在");
        }
        GroupChatPlanDTO plan = linkResp.getResult();
        plan.getStaffIdList();
        String content = plan.getChatContent();
        JSONArray array = JSONArray.parseArray(content);
        List<String> materialIdList = new ArrayList<>();
        for (int i = 0; i < array.size(); i++) {
            JSONObject json = array.getJSONObject(i);
            materialIdList.add(json.getString("relation_id"));
        }
        // 查询素材列表
        GroupChatPlanVO vo = EntityUtil.changeEntityByJSON(GroupChatPlanVO.class, plan);
        List<MaterialDTO> materials = materialApiService.listMaterialByIds(materialIdList);
        vo.setMaterialIdList(materials);
        GroupChatPlanSearchQDTO qdto = new GroupChatPlanSearchQDTO();
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        qdto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        qdto.setEnterpriseId(loginUser.getEnterpriseId());
        BasePageInfo basePageInfo = new BasePageInfo();
        basePageInfo.setPageNum(1);
        basePageInfo.setPageSize(10000);
        ServiceResponse<Page<GroupChatPlanOwnerLogDTO>> ownerResp = this.groupChatPlanApiService
                .listOwnerLogPage(planId, qdto, basePageInfo);
        if (ownerResp.isSuccess()) {
            List<GroupChatPlanOwnerLogDTO> list = ownerResp.getResult().getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                List<ChatOwnerVO> ownerList = list.stream().map(dto -> {
                    ChatOwnerVO owner = new ChatOwnerVO();
                    owner.setStaffId(dto.getStaffId());
                    owner.setStaffName(dto.getStaffName());
                    return owner;
                }).collect(Collectors.toList());
                vo.setOwnerList(ownerList);
            }
        }
        return RestResponse.successResult(vo);
    }

    @RequestMapping("owner-log")
    public RestResponse<Object> ownerLog(Long planId, GroupChatPlanSearchQDTO qdto, BasePageInfo basePageInfo) {
        if (null == planId) {
            return RestResponse.failure("9999", "计划ID空");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        qdto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        ServiceResponse<Page<GroupChatPlanOwnerLogDTO>> resp = this.groupChatPlanApiService.listOwnerLogPage(planId,
                qdto, basePageInfo);
        if (!resp.isSuccess()) {
            return RestResponse.failure("1", "计划不存在");
        }
        return RestResponse.successResult(resp.getResult());
    }

    @RequestMapping("log")
    public RestResponse<Object> log(Long planId, GroupChatPlanSearchQDTO qdto, BasePageInfo basePageInfo) {
        if (null == planId) {
            return RestResponse.failure("9999", "计划ID空");
        }
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        qdto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        ServiceResponse<Page<GroupChatPlanLogDTO>> resp = this.groupChatPlanApiService.listLogPage(planId, qdto,
                basePageInfo);
        if (!resp.isSuccess()) {
            return RestResponse.failure("1", "计划不存在");
        }
        return RestResponse.successResult(resp.getResult());
    }

}
